"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("@fluentui/react");
const react_2 = require("react");
const FluentDialog_1 = require("./FluentDialog");
const react_3 = __importDefault(require("react"));
const formbuilder_1 = require("@wiberg/formbuilder");
const __1 = require("../..");
const FluentList = (props) => {
    var _a, _b, _c, _d, _e;
    const editorFormRef = (0, react_2.useRef)();
    const [items, setItems] = (0, react_2.useState)(props.items);
    const [showEditor, setShowEditor] = (0, react_2.useState)(false);
    const [columns, setColumns, columnRef] = (0, formbuilder_1.useStateRef)();
    const [showValidationOverrideConfirm, setShowValidationOverrideConfirm] = (0, react_2.useState)(false);
    const [enableShimmer, setEnableShimmer] = (0, react_2.useState)(false);
    const [newItemMode, setNewItemMode] = (0, react_2.useState)(false);
    const [menuItems, setMenuItems] = (0, react_2.useState)(undefined);
    const [selection] = (0, react_2.useState)(new react_1.Selection({
        selectionMode: props.listProps.config.multiSelect ? react_1.SelectionMode.multiple : react_1.SelectionMode.single,
        getKey: (item, index) => props.listProps.config.itemIdentifier(item),
        onSelectionChanged: () => {
            var _a;
            props.updateSelectedItems((_a = selection.getSelection()) !== null && _a !== void 0 ? _a : []);
        }
    }));
    (0, react_2.useEffect)(() => {
        let cols = buildColumns();
        setColumns(cols);
    }, [props.columns]);
    (0, react_2.useEffect)(() => {
        var _a;
        if (!!((_a = props.listProps.config) === null || _a === void 0 ? void 0 : _a.shimmerLines) && !!!(items === null || items === void 0 ? void 0 : items.length)) {
            setEnableShimmer(true);
            setTimeout(() => {
                setEnableShimmer(false);
            }, 5000);
        }
        else
            setEnableShimmer(false);
    }, [items, (_a = props.listProps.config) === null || _a === void 0 ? void 0 : _a.shimmerLines]);
    (0, react_2.useEffect)(() => {
        setItems(props.items);
    }, [props.items]);
    (0, react_2.useEffect)(() => {
        var _a, _b;
        if (((_a = props.listProps.menuConfig) === null || _a === void 0 ? void 0 : _a.actions) == null)
            return;
        let actions = (_b = props.listProps.menuConfig) === null || _b === void 0 ? void 0 : _b.actions(createItem, editItem, deleteItems);
        if (!(actions === null || actions === void 0 ? void 0 : actions.length))
            return; // is it needed to set to empty array as well? (ewi)
        // parse to commandbarProps
        let commandBarItems = actions.map(_ => {
            var _a;
            return {
                key: _.title,
                text: _.title,
                onClick: () => _.action(props.selectedItems),
                disabled: formbuilder_1.formListHelper.isMenuItemDisabled(_.selectionMode, (_a = props.selectedItems) === null || _a === void 0 ? void 0 : _a.length),
                iconProps: { iconName: _.iconName }
            };
        });
        setMenuItems(commandBarItems);
    }, [props.listProps.menuConfig, props.selectedItems]);
    const buildColumns = () => {
        return props.columns.map(_ => {
            let col = _;
            col.onColumnClick = () => props.sortColumn(_);
            return col;
        });
    };
    // TODO: move crud operations out to FormList (ewi)
    //       with alot of other stuff as well
    const createItem = (pre) => {
        let item = {};
        if (pre) {
            let result = pre(item);
            if (result === false)
                return;
        }
        selection.setAllSelected(false);
        props.updateSelectedItems([item]);
        setNewItemMode(true);
        setShowEditor(true);
    };
    const editItem = (pre) => {
        let sItem = Object.assign({}, props.selectedItems[0]);
        if (pre) {
            let result = pre(sItem);
            if (result === false)
                return;
        }
        props.updateSelectedItems([sItem]);
        setTimeout(() => {
            setNewItemMode(false);
            setShowEditor(true);
        }, 200); // TODO: why was this needed ? (ewi)
    };
    const deleteItems = (pre) => {
        const success = props.deleteItems(pre);
        if (!success)
            return;
        selection.setAllSelected(false);
    };
    const validateForm = () => __awaiter(void 0, void 0, void 0, function* () {
        var _f;
        let validated = yield ((_f = editorFormRef.current) === null || _f === void 0 ? void 0 : _f.validateItem());
        return validated;
    });
    const saveForm = () => __awaiter(void 0, void 0, void 0, function* () {
        var _g;
        let validated = yield validateForm();
        if (validated == formbuilder_1.ValidationResult.FailedDontBlock) {
            setShowValidationOverrideConfirm(true);
        }
        else if (validated == formbuilder_1.ValidationResult.Success) {
            props.onItemChange((_g = editorFormRef.current) === null || _g === void 0 ? void 0 : _g.getItem());
            setShowEditor(false);
        }
    });
    const dismissForm = () => __awaiter(void 0, void 0, void 0, function* () {
        if (newItemMode)
            props.updateSelectedItems([]);
        setNewItemMode(false);
        setShowEditor(false);
    });
    const getItem = () => {
        var _a;
        return (_a = editorFormRef.current) === null || _a === void 0 ? void 0 : _a.getItem();
    };
    const onRenderFooterContent = (0, react_2.useCallback)(() => {
        var _a, _b;
        if (((_b = (_a = props.listProps) === null || _a === void 0 ? void 0 : _a.editorConfig) === null || _b === void 0 ? void 0 : _b.customFooter) != undefined)
            return props.listProps.editorConfig.customFooter(saveForm, dismissForm, validateForm, getItem);
        else
            return (react_3.default.createElement("div", null,
                react_3.default.createElement(react_1.PrimaryButton, { text: newItemMode ? formbuilder_1.lang.texts.areas.common.create : formbuilder_1.lang.texts.areas.common.save, styles: { root: { marginRight: 8 } }, onClick: saveForm }),
                react_3.default.createElement(react_1.DefaultButton, { text: formbuilder_1.lang.texts.areas.common.cancel, onClick: dismissForm })));
    }, [showEditor]);
    const handleColumnReorder = (draggedIndex, targetIndex) => {
        if (columns == null)
            return;
        const draggedItems = columns[draggedIndex];
        const newColumns = [...columns];
        // insert before the dropped item
        newColumns.splice(draggedIndex, 1);
        newColumns.splice(targetIndex, 0, draggedItems);
        setColumns(newColumns);
    };
    return react_3.default.createElement(react_3.default.Fragment, null,
        (showEditor && ((_b = props.selectedItems) === null || _b === void 0 ? void 0 : _b.length))
            && react_3.default.createElement(FluentDialog_1.FluentDialog, { item: props.selectedItems[0], editorConfig: props.listProps.editorConfig, formListProps: props, keyPrefix: (_c = props.keyPrefix) !== null && _c !== void 0 ? _c : "", newItemMode: newItemMode, onRenderFooterContent: onRenderFooterContent, show: (show) => setShowEditor(show), dialogType: (_d = props.listProps.editorConfig) === null || _d === void 0 ? void 0 : _d.type, ref: editorFormRef }),
        react_3.default.createElement("div", { className: "formbuilder-listcontainer-toolbar" },
            ((_e = props.listProps.searchConfig) === null || _e === void 0 ? void 0 : _e.searchEnabled) &&
                react_3.default.createElement(__1.Searcher, { items: items, onSearch: (filteredItems, searchText) => {
                        props.updateFilteredItems(filteredItems);
                        selection.setAllSelected(false);
                        props.updateSelectedItems([]);
                    }, className: "tp-searcher", delay: 600, searchPlaceHolder: props.listProps.searchConfig.searchPlaceHolder, enableTextHighlight: true, searchFields: props.listProps.searchConfig.searchableFields, highLightElements: () => Array.from(document.querySelectorAll('.ms-List .ms-DetailsRow-cell:not(.ms-DetailsRow-cellCheck)'))
                        .filter(_ => {
                        var _a;
                        if (((_a = props.listProps.searchConfig) === null || _a === void 0 ? void 0 : _a.searchableFields) == null)
                            return [];
                        return props.listProps.searchConfig.searchableFields.findIndex(sf => sf == _.attributes["data-automation-key"].value) >= 0;
                    }), filterOnItemsChange: true }),
            menuItems !== undefined &&
                react_3.default.createElement(react_1.CommandBar, { items: menuItems, styles: { root: { padding: 0 } } })),
        react_3.default.createElement("div", { className: "formbuilder-listcontainer-list" },
            react_3.default.createElement(react_1.ShimmeredDetailsList, { columns: columns, items: props.filteredItems, 
                // selectionPreservedOnEmptyClick
                selection: selection, enableShimmer: enableShimmer, shimmerLines: props.listProps.config.shimmerLines, layoutMode: react_1.DetailsListLayoutMode.fixedColumns, constrainMode: react_1.ConstrainMode.horizontalConstrained, setKey: 'itemId', getKey: (item) => `item-${props.listProps.config.itemIdentifier(item)}`, onItemInvoked: props.listProps.config.disableItemInvoke ? undefined :
                    (item, index, ev) => {
                        if (item == null) {
                            props.updateSelectedItems([]);
                        }
                        else {
                            props.updateSelectedItems([item]);
                        }
                        editItem();
                    }, columnReorderOptions: (columns === null || columns === void 0 ? void 0 : columns.length) ? { handleColumnReorder } : undefined })),
        (!enableShimmer && !!!(items === null || items === void 0 ? void 0 : items.length)) && react_3.default.createElement(react_1.Label, null, "No items found"),
        react_3.default.createElement(react_1.Dialog, { hidden: !showValidationOverrideConfirm, title: formbuilder_1.lang.texts.areas.list.validationRequirementsNotMet.title, onDismiss: () => setShowValidationOverrideConfirm(false), dialogContentProps: {
                subText: formbuilder_1.lang.texts.areas.list.validationRequirementsNotMet.desctiption
            }, modalProps: {
                isBlocking: true
            } },
            react_3.default.createElement(react_1.DialogFooter, null,
                react_3.default.createElement(react_1.PrimaryButton, { text: formbuilder_1.lang.texts.areas.list.validationRequirementsNotMet.ok, onClick: () => {
                        var _a;
                        setShowValidationOverrideConfirm(false);
                        setShowEditor(false);
                        props.onItemChange((_a = editorFormRef.current) === null || _a === void 0 ? void 0 : _a.getItem());
                    } }),
                react_3.default.createElement(react_1.DefaultButton, { text: formbuilder_1.lang.texts.areas.list.validationRequirementsNotMet.cancel, onClick: () => setShowValidationOverrideConfirm(false) }))));
};
exports.default = FluentList;
//# sourceMappingURL=FluentList.js.map